@@ -5,8 +5,8 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 1.0
-;; Package-Requires: ((emacs "27.1") (compat "29.1.4.2"))
+;; Version: 1.1
+;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4"))
 ;; Homepage: https://github.com/minad/cape
 ;; Keywords: abbrev, convenience, matching, completion, wp
 
@@ -88,16 +88,16 @@ See `dabbrev-case-fold-search' for details."
                  (other :tag "on" t)))
 
 (defcustom cape-dabbrev-min-length 4
-  "Minimum length of dabbrev expansions.
+  "Minimum length of Dabbrev expansions.
 This setting ensures that words which are too short
 are not offered as completion candidates, such that
 auto completion does not pop up too aggressively."
   :type 'natnum)
 
 (defcustom cape-dabbrev-check-other-buffers t
-  "Buffers to check for dabbrev.
+  "Buffers to check for Dabbrev.
 
-If t, check all other buffers (subject to dabbrev ignore rules).
+If t, check all other buffers (subject to Dabbrev ignore rules).
 Any other non-nil value only checks some other buffers, as per
 `dabbrev-select-buffers-function'."
   :type '(choice (const :tag "off" nil)
@@ -158,6 +158,17 @@ The buffers are scanned for completion candidates by `cape-line'."
                   (replace-match str nil nil input))))
       str))
 
+(defun cape--separator-p (str)
+  "Return non-nil if input STR has a separator character.
+Separator characters are used by completion styles like Orderless
+to split filter words.  In Corfu, the separator is configurable
+via the variable `corfu-separator'."
+  (string-search (string ;; Support `corfu-separator' and Orderless
+                  (or (and (bound-and-true-p corfu-mode)
+                           (bound-and-true-p corfu-separator))
+                      ?\s))
+                 str))
+
 (defmacro cape--silent (&rest body)
   "Silence BODY."
   (declare (indent 0))
@@ -267,16 +278,19 @@ NAME is the name of the Capf, BEG and END are the input markers."
   "Create completion TABLE with properties.
 CATEGORY is the optional completion category.
 SORT should be nil to disable sorting."
-  (if (or (not table) (and (not category) sort))
-      table
-    (let ((metadata `(metadata
-                      ,@(and category `((category . ,category)))
-                      ,@(and (not sort) '((display-sort-function . identity)
-                                          (cycle-sort-function . identity))))))
-      (lambda (str pred action)
-        (if (eq action 'metadata)
-            metadata
-          (complete-with-action action table str pred))))))
+  ;; The metadata will be overridden if the category is non-nil, if the table is
+  ;; a function table or if sorting should be disabled for a non-nil
+  ;; non-function table.
+  (if (or category (functionp table) (and (not sort) table))
+      (let ((metadata `(metadata
+                        ,@(and category `((category . ,category)))
+                        ,@(and (not sort) '((display-sort-function . identity)
+                                            (cycle-sort-function . identity))))))
+        (lambda (str pred action)
+          (if (eq action 'metadata)
+              metadata
+            (complete-with-action action table str pred))))
+    table))
 
 (defun cape--dynamic-table (beg end fun)
   "Create dynamic completion table from FUN with caching.
@@ -311,9 +325,9 @@ string as first argument to the completion table."
       ;; a thing.
       (unless (or (eq action 'metadata) (eq (car-safe action) 'boundaries))
         (let ((input (buffer-substring-no-properties beg end)))
-          (when (or (not valid)
-                    (not (or (string-match-p "\\s-" input) ;; Support Orderless
-                             (funcall valid input))))
+          (unless (and valid
+                       (or (cape--separator-p input)
+                           (funcall valid input)))
             (let* (;; Reset in case `all-completions' is used inside FUN
                    completion-ignore-case completion-regexp-list
                    ;; Retrieve new state by calling FUN
@@ -387,7 +401,6 @@ If INTERACTIVE is nil the function acts like a Capf."
                                       ('nil default-directory)
                                       ((pred stringp) cape-file-directory)
                                       (_ (funcall cape-file-directory))))
-                 ;; TODO: Bounds are not correct in comint/eshell buffers
                  (`(,beg . ,end) (cape--bounds 'filename))
                  (non-essential t)
                  (file (buffer-substring-no-properties beg end))
@@ -520,7 +533,7 @@ If INTERACTIVE is nil the function acts like a Capf."
 (declare-function dabbrev--reset-global-variables "dabbrev")
 
 (defun cape--dabbrev-list (input)
-  "Find all dabbrev expansions for INPUT."
+  "Find all Dabbrev expansions for INPUT."
   (cape--silent
     (let ((dabbrev-check-other-buffers (not (null cape-dabbrev-check-other-buffers)))
           (dabbrev-check-all-buffers (eq cape-dabbrev-check-other-buffers t)))
@@ -560,7 +573,7 @@ If INTERACTIVE is nil the function acts like a Capf."
   "Complete with Dabbrev at point.
 
 If INTERACTIVE is nil the function acts like a Capf.  In case you
-observe a performance issue with autocompletion and `cape-dabbrev'
+observe a performance issue with auto-completion and `cape-dabbrev'
 it is strongly recommended to disable scanning in other buffers.
 See the user options `cape-dabbrev-min-length' and
 `cape-dabbrev-check-other-buffers'."
@@ -784,10 +797,9 @@ changed.  The function `cape-company-to-capf' is experimental."
     (when-let ((prefix (cape--company-call backend 'prefix))
                (initial-input (if (stringp prefix) prefix (car-safe prefix))))
       (let* ((end (point)) (beg (- end (length initial-input)))
-             (dups (cape--company-call backend 'duplicates))
              (valid (if (cape--company-call backend 'no-cache initial-input)
                         #'equal (or valid #'string-prefix-p)))
-             candidates)
+             restore-props)
         (list beg end
               (funcall
                (if (cape--company-call backend 'ignore-case)
@@ -797,9 +809,14 @@ changed.  The function `cape-company-to-capf' is experimental."
                 (cape--dynamic-table
                  beg end
                  (lambda (input)
-                   (setq candidates (cape--company-call backend 'candidates input))
-                   (when dups (setq candidates (delete-dups candidates)))
-                   (cons (apply-partially valid input) candidates)))
+                   (let ((cands (cape--company-call backend 'candidates input)))
+                     ;; The candidate string including text properties should be
+                     ;; restored in the :exit-function, if the UI does not
+                     ;; guarantee this itself.  Restoration is not necessary for
+                     ;; Corfu since the introduction of `corfu--exit-function'.
+                     (unless (and (bound-and-true-p corfu-mode) (fboundp 'corfu--exit-function))
+                       (setq restore-props cands))
+                     (cons (apply-partially valid input) cands))))
                 :category backend
                 :sort (not (cape--company-call backend 'sorted))))
               :exclusive 'no
@@ -811,13 +828,13 @@ changed.  The function `cape-company-to-capf' is experimental."
               :company-kind (lambda (x) (cape--company-call backend 'kind x))
               :annotation-function (lambda (x)
                                      (when-let (ann (cape--company-call backend 'annotation x))
-                                       (if (string-match-p "^[ \t]" ann)
-                                           ann
-                                         (concat " " ann))))
-              :exit-function
-              (lambda (x _status)
-                (cape--company-call backend 'post-completion
-                                    (or (car (member x candidates)) x))))))))
+                                       (concat " " (string-trim ann))))
+              :exit-function (lambda (x _status)
+                               ;; Restore the candidate string including
+                               ;; properties if restore-props is non-nil.  See
+                               ;; the comment above.
+                               (setq x (or (car (member x restore-props)) x))
+                               (cape--company-call backend 'post-completion x)))))))
 
 ;;;###autoload
 (defun cape-interactive (&rest capfs)
@@ -849,7 +866,7 @@ changed.  The function `cape-company-to-capf' is experimental."
 The functions `cape-wrap-super' and `cape-capf-super' are experimental."
   (when-let ((results (delq nil (mapcar #'funcall capfs))))
     (pcase-let* ((`((,beg ,end . ,_)) results)
-                 (cand-ht (make-hash-table :test #'equal))
+                 (cand-ht nil)
                  (tables nil)
                  (prefix-len nil))
       (cl-loop for (beg2 end2 . rest) in results do
@@ -885,14 +902,15 @@ The functions `cape-wrap-super' and `cape-capf-super' are experimental."
                                 (sort (or (completion-metadata-get md 'display-sort-function)
                                           #'identity))
                                 (cands (funcall sort (all-completions str table pr))))
-                           (cl-loop for cell on cands
-                                    for cand = (car cell) do
-                                    (if (eq (gethash cand ht t) t)
-                                        (puthash cand plist ht)
-                                      (setcar cell nil)))
+                           (cl-loop
+                            for cand in-ref cands do
+                            (if (eq (gethash cand ht t) t)
+                                (puthash cand plist ht)
+                              (setf cand (propertize cand #'cape-capf-super
+                                                     (cons cand plist)))))
                            (push cands candidates)))
                 (setq cand-ht ht)
-                (delq nil (apply #'nconc (nreverse candidates)))))
+                (apply #'nconc (nreverse candidates))))
              (_ ;; try-completion and test-completion
               (cl-loop for (table . plist) in tables thereis
                        (complete-with-action
@@ -907,8 +925,12 @@ The functions `cape-wrap-super' and `cape-capf-super' are experimental."
         ,@(mapcan
            (lambda (prop)
              (list prop (lambda (cand &rest args)
-                          (when-let (fun (plist-get (gethash cand cand-ht) prop))
-                            (apply fun cand args)))))
+                          (let ((ref (get-text-property 0 #'cape-capf-super cand)))
+                            (when-let ((fun (plist-get
+                                             (or (cdr ref)
+                                                 (and cand-ht (gethash cand cand-ht)))
+                                             prop)))
+                              (apply fun (or (car ref) cand) args))))))
            '(:company-docsig :company-location :company-kind
              :company-doc-buffer :company-deprecated
              :annotation-function :exit-function))))))
@@ -976,7 +998,7 @@ completion table is refreshed on every input change."
                (input (buffer-substring-no-properties beg end)))
           (lambda (str pred action)
             (let ((new-input (buffer-substring-no-properties beg end)))
-              (unless (or (string-match-p "\\s-" new-input) ;; Support Orderless
+              (unless (or (cape--separator-p new-input)
                           (funcall valid input new-input))
                 (pcase
                     ;; Reset in case `all-completions' is used inside CAPF
@@ -1127,7 +1149,7 @@ This function can be used as an advice around an existing Capf."
      `(,beg ,end ,(cape--accept-all-table table) . ,plist))))
 
 (defmacro cape--capf-wrapper (wrapper)
-  "Create a capf transformer from WRAPPER."
+  "Create a Capf transformer from WRAPPER."
   `(defun ,(intern (format "cape-capf-%s" wrapper)) (&rest args)
      (lambda () (apply #',(intern (format "cape-wrap-%s" wrapper)) args))))
 
@@ -1164,14 +1186,5 @@ This function can be used as an advice around an existing Capf."
 ;;;###autoload (autoload 'cape-capf-silent "cape")
 (cape--capf-wrapper silent)
 
-;;;###autoload
-(define-obsolete-function-alias 'cape-interactive-capf #'cape-capf-interactive "0.17")
-
-;;;###autoload
-(define-obsolete-function-alias 'cape-super-capf #'cape-capf-super "0.17")
-
-;;;###autoload
-(define-obsolete-function-alias 'cape-symbol #'cape-elisp-symbol "0.17")
-
 (provide 'cape)
 ;;; cape.el ends here
